Public Class Form1

    Private m_DataDirty As Boolean
    Private m_FileName As String
    Private WithEvents m_MruList As MruList

    ' Make a new MruList object.
    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        m_MruList = New MruList(Application.ProductName, mnuFile, 4)
    End Sub

    ' Open a file.
    Private Sub mnuFileOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileOpen.Click
        ' Make sure the current data is safe.
        If Not DataSafe() Then Exit Sub

        If dlgOpen.ShowDialog() = Windows.Forms.DialogResult.OK Then
            LoadData(dlgOpen.FileName)
        End If
    End Sub

    ' Return True if it is safe to discard the current data.
    Private Function DataSafe() As Boolean
        If Not m_DataDirty Then Return True

        Select Case MessageBox.Show( _
                "The data has been modified. Do you want to save the changes?", _
                "Save Changes?", MessageBoxButtons.YesNoCancel)
            Case Windows.Forms.DialogResult.Cancel
                ' The user is canceling the operation. 
                ' Don't discard the changes.
                Return False
            Case Windows.Forms.DialogResult.No
                ' The user wants to discard the changes.
                Return True
            Case Windows.Forms.DialogResult.Yes
                ' Try to save the data.
                mnuFileSave_Click(Nothing, Nothing)

                ' See if the data was saved.
                Return (Not m_DataDirty)
        End Select
    End Function

    ' Load a data file.
    Private Sub LoadData(ByVal file_name As String)
        Dim stream_reader As IO.StreamReader = Nothing
        Try
            ' Load the file.
            stream_reader = New IO.StreamReader(file_name)
            Dim txt As String = stream_reader.ReadToEnd()
            rchFile.Text = txt

            ' Save the file name and title.
            m_FileName = file_name
            Me.Text = Application.ProductName & " [" & FileTitle(m_FileName) & "]"
            m_DataDirty = False

            ' Update the MRU list.
            m_MruList.Add(m_FileName)
            mnuFileSave.Enabled = False
            ' mnuFileSaveAs.Enabled = False

        Catch ex As Exception
            MessageBox.Show("Error loading file " & file_name & _
                vbCrLf & ex.Message, _
                "Load Error", _
                MessageBoxButtons.OK, _
                MessageBoxIcon.Error)
            m_MruList.Remove(file_name)
        Finally
            If stream_reader IsNot Nothing Then
                stream_reader.Close()
                stream_reader.Dispose()
            End If
        End Try
    End Sub

    ' Save the file.
    Private Sub SaveData(ByVal file_name As String)
        Dim stream_writer As IO.StreamWriter
        Try
            ' Save the file.
            stream_writer = New IO.StreamWriter(file_name)
            stream_writer.Write(Me.rchFile.Text)
            stream_writer.Close()

            ' Save the file name and title.
            m_FileName = file_name
            Me.Text = Application.ProductName & " [" & FileTitle(file_name) & "]"
            m_DataDirty = False

            ' Update the MRU list.
            m_MruList.Add(m_FileName)
            Me.mnuFileSave.Enabled = False
            ' Me.mnuFileSaveAs.Enabled = False
        Catch ex As Exception
            MessageBox.Show("Error saving file " & file_name & _
                vbCrLf & ex.Message, _
                "Save Error", _
                MessageBoxButtons.OK, _
                MessageBoxIcon.Error)
        End Try
    End Sub

    ' Mark the data as modified.
    Private Sub rchFile_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles rchFile.TextChanged
        If Not m_DataDirty Then
            If m_FileName IsNot Nothing Then
                Me.Text = Application.ProductName & "*[" & FileTitle(m_FileName) & "]"
            End If
            m_DataDirty = True
            mnuFileSave.Enabled = True
            mnuFileSaveAs.Enabled = True
        End If
    End Sub

    ' Open a file selected from the MRU list.
    Private Sub m_MruList_OpenFile(ByVal file_name As String) Handles m_MruList.OpenFile
        LoadData(file_name)
    End Sub

    ' Save the file.
    Private Sub mnuFileSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileSave.Click
        If m_FileName Is Nothing Then
            mnuFileSaveAs_Click(Nothing, Nothing)
        Else
            SaveData(m_FileName)
        End If
    End Sub

    ' Save the file with a new name.
    Private Sub mnuFileSaveAs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileSaveAs.Click
        dlgSave.FileName = dlgOpen.FileName
        If dlgSave.ShowDialog() = Windows.Forms.DialogResult.OK Then
            SaveData(dlgSave.FileName)
        End If
    End Sub

    ' Close the application.
    Private Sub mnuFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileExit.Click
        Me.Close()
    End Sub

    ' Don't close if the data's not safe.
    Private Sub Form1_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles MyBase.Closing
        e.Cancel = Not DataSafe()
    End Sub

    ' Start a new document.
    Private Sub mnuFileNew_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileNew.Click
        ' Make sure the current data is safe.
        If Not DataSafe() Then Exit Sub

        Me.rchFile.Text = ""
        m_FileName = Nothing
        Me.Text = Application.ProductName & " []"
        m_DataDirty = False

        ' No point in saving a blank file.
        Me.mnuFileSave.Enabled = False
        Me.mnuFileSaveAs.Enabled = False
    End Sub
End Class
