Imports System.IO

Public Class Form1
    Private Sub mnuFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileExit.Click
        Me.Close()
    End Sub

    Private Sub mnuFileOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileOpen.Click
        If dlgOpen.ShowDialog() = Windows.Forms.DialogResult.OK Then
            ' Make a FileProcessorFactory.
            Dim fp_factory As New FileProcessorFactory()

            ' Make a FileProcessor.
            Dim fp As FileProcessor = fp_factory.MakeFileProcessor(dlgOpen.FileName)

            ' Tell the FileProcessor to do its thing.
            fp.ProcessFile()
        End If
    End Sub
End Class

' Make various kinds of FileProcessor objects.
Public Class FileProcessorFactory
    Public Function MakeFileProcessor(ByVal file_name As String) As FileProcessor
        Dim file_info As New FileInfo(file_name)

        ' See what kind of file this is.
        Dim file_processor As FileProcessor = Nothing
        Select Case file_info.Extension.ToLower
            Case ".bmp"
                file_processor = New BitmapProcessor(file_info)
            Case ".txt"
                file_processor = New TextFileProcessor(file_info)
            Case ".mdb"
                file_processor = New MdbProcessor(file_info)
            Case Else
                file_processor = New OtherProcessor(file_info)
        End Select

        Return file_processor
    End Function
End Class

' Base class for file processors.
Public MustInherit Class FileProcessor
    ' The file we will process.
    Protected m_FileInfo As FileInfo

    Protected Sub New(ByVal file_info As FileInfo)
        m_FileInfo = file_info
    End Sub

    ' Process the file appropriately.
    Public MustOverride Sub ProcessFile()
End Class

' Processor for bitmap files.
Public Class BitmapProcessor
    Inherits FileProcessor
    Public Sub New(ByVal file_info As FileInfo)
        MyBase.New(file_info)
    End Sub

    ' Process the file by displaying the bitmap in a dialog.
    Public Overrides Sub ProcessFile()
        ' Display the file name.
        Dim dlg As New dlgShowBitmap
        dlg.Text = m_FileInfo.Name

        ' Display the Bitmap.
        Dim bm As Bitmap = DirectCast(New Bitmap(m_FileInfo.FullName).Clone, Bitmap)
        dlg.picBitmap.Image = bm

        ' Show the dialog.
        dlg.ShowDialog()
    End Sub
End Class

' Processor for text files.
Public Class TextFileProcessor
    Inherits FileProcessor
    Public Sub New(ByVal file_info As FileInfo)
        MyBase.New(file_info)
    End Sub

    ' Process the file by displaying the text in a dialog.
    Public Overrides Sub ProcessFile()
        MessageBox.Show( _
            My.Computer.FileSystem.ReadAllText(m_FileInfo.FullName), _
            m_FileInfo.Name, MessageBoxButtons.OK, _
            MessageBoxIcon.Information)
    End Sub
End Class

' Processor for Access database files.
Public Class MdbProcessor
    Inherits FileProcessor
    Public Sub New(ByVal file_info As FileInfo)
        MyBase.New(file_info)
    End Sub

    ' Process the file by displaying a message box.
    Public Overrides Sub ProcessFile()
        MessageBox.Show("Read the database here" & _
            vbCrLf & m_FileInfo.FullName, _
            "Read Data Here", _
            MessageBoxButtons.OK, MessageBoxIcon.Information)
    End Sub
End Class

' Processor for other files.
Public Class OtherProcessor
    Inherits FileProcessor
    Public Sub New(ByVal file_info As FileInfo)
        MyBase.New(file_info)
    End Sub

    ' Process the file by displaying a message box.
    Public Overrides Sub ProcessFile()
        MessageBox.Show("Unknown extension " & m_FileInfo.Extension & _
            vbCrLf & m_FileInfo.FullName, _
            "Unkown File Type", _
            MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
    End Sub
End Class