Imports System.Data.OleDb

Public Class dlgCriteria
    ' The DataGridView on the main form.
    Public TheDataGridView As DataGridView = Nothing

    ' The collection of criteria.
    Public TheCriteria As Collection = Nothing

    ' Delimiters for the field choices.
    Private m_Delimiters As Collection

    ' Load the list of database fields.
    Private Sub dlgSelectFields_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ' Make sure TheDataGridView and TheCriteria have been initialized.
        Debug.Assert(TheDataGridView IsNot Nothing, "TheDataGridView is Nothing")
        Debug.Assert(TheCriteria IsNot Nothing, "TheCriteria is Nothing")

        ' Get the cell's template object.
        Dim dgv_cell As DataGridViewCell = dgvCriteria.Columns(0).CellTemplate
        Dim combo_cell As DataGridViewComboBoxCell = DirectCast(dgv_cell, DataGridViewComboBoxCell)

        ' Make a list of the fields.
        combo_cell.Items.Clear()
        m_Delimiters = New Collection
        For i As Integer = 0 To TheDataGridView.Columns.Count - 1
            ' Add the name to the combo cell.
            Dim field_name As String = TheDataGridView.Columns(i).Name
            combo_cell.Items.Add(field_name)

            ' Get an appropriate delimiter for the data type.
            Dim delimiter As String = ""
            If TheDataGridView.Columns(i).ValueType Is GetType(String) Then
                delimiter = "'"
            ElseIf TheDataGridView.Columns(i).ValueType Is GetType(Date) Then
                ' Note that you need to handle dates differently in SQL Server.
                delimiter = "#"
            End If
            m_Delimiters.Add(delimiter, field_name)
        Next i

        ' Display current criteria.
        dgvCriteria.RowCount = TheCriteria.Count + 1
        For r As Integer = 0 To TheCriteria.Count - 1
            Dim condition As Criterion = DirectCast(TheCriteria(r + 1), Criterion)

            dgvCriteria.Rows(r).Cells(0).Value = condition.FieldName
            dgvCriteria.Rows(r).Cells(1).Value = condition.Op
            dgvCriteria.Rows(r).Cells(2).Value = condition.Value
        Next r
    End Sub

    ' Create the new Criteria collection.
    Private Sub btnOk_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOk.Click
        ' Verify that each row has a field and operator.
        For r As Integer = 0 To dgvCriteria.RowCount - 2
            If dgvCriteria.Rows(r).Cells(0).Value Is Nothing Then
                MessageBox.Show( _
                    "You must select a field for every row", _
                    "Missing Field", _
                    MessageBoxButtons.OK, _
                    MessageBoxIcon.Exclamation)
                dgvCriteria.CurrentCell = dgvCriteria.Rows(r).Cells(0)
                dgvCriteria.Select()
                Me.DialogResult = Windows.Forms.DialogResult.None
                Exit Sub
            End If
            If dgvCriteria.Rows(r).Cells(1).Value Is Nothing Then
                MessageBox.Show( _
                    "You must select an operator for every row", _
                    "Missing Field", _
                    MessageBoxButtons.OK, _
                    MessageBoxIcon.Exclamation)
                dgvCriteria.CurrentCell = dgvCriteria.Rows(r).Cells(1)
                dgvCriteria.Select()
                Me.DialogResult = Windows.Forms.DialogResult.None
                Exit Sub
            End If
        Next r

        ' Make the new criteria collection.
        TheCriteria = New Collection
        For r As Integer = 0 To dgvCriteria.RowCount - 2
            Dim field_name As String = _
                DirectCast(dgvCriteria.Rows(r).Cells(0).Value, String)
            Dim delimiter As String = _
                DirectCast(m_Delimiters(field_name), String)
            Dim op As String = _
                DirectCast(dgvCriteria.Rows(r).Cells(1).Value, String)
            Dim value As String = _
                DirectCast(dgvCriteria.Rows(r).Cells(2).Value, String)

            TheCriteria.Add(New Criterion(field_name, op, value, delimiter))
        Next r
    End Sub
End Class

' Represent a condition such as FirstName >= 'Stephens'.
Public Class Criterion
    Public FieldName As String
    Public Op As String
    Public Value As String
    Public Delimiter As String

    Public Sub New(ByVal new_field_name As String, ByVal new_op As String, ByVal new_value As String, ByVal new_delimiter As String)
        FieldName = new_field_name
        Op = new_op
        Value = new_value
        Delimiter = new_delimiter
    End Sub

    ' Initialize with tab-delimited values.
    Public Sub New(ByVal txt As String)
        Dim items() As String = txt.Split(CChar(vbTab))
        FieldName = items(0)
        Op = items(1)
        Value = items(2)
        Delimiter = items(3)
    End Sub

    ' Return tab-delimited values.
    Public Overrides Function ToString() As String
        Return FieldName & vbTab & Op & vbTab & Value & vbTab & Delimiter
    End Function
End Class
