<ToolboxBitmap(GetType(StyledListBox), "tmbStyledListBox")> _
Public Class StyledListBox
    Inherits ListBox

    ' Create a new strongly typed Items property so 
    ' the Properties Window can provide an editor for it.
    ' Delegate it to the inherited Items property.
    Public Shadows Property Items() As StyledListBoxItem()
        Get
            Dim item_list(0 To MyBase.Items.Count - 1) As StyledListBoxItem
            For i As Integer = 0 To MyBase.Items.Count - 1
                item_list(i) = DirectCast(MyBase.Items(i), StyledListBoxItem)
            Next i
            Return item_list
        End Get
        Set(ByVal value() As StyledListBoxItem)
            MyBase.Items.Clear()
            For Each styled_item As StyledListBoxItem In value
                MyBase.Items.Add(styled_item)
            Next styled_item
        End Set
    End Property

    ' Specify OwnerDraw mode.
    Public Sub New()
        MyBase.New()
        MyBase.DrawMode = Windows.Forms.DrawMode.OwnerDrawVariable
    End Sub

    ' Set the size for an item.
    Private Const ITEM_MARGIN As Integer = 2

    Private Sub StyledListBox_MeasureItem(ByVal sender As Object, ByVal e As System.Windows.Forms.MeasureItemEventArgs) Handles Me.MeasureItem
        If e.Index < 0 Or e.Index >= MyBase.Items.Count Then Exit Sub

        ' Convert the item into a StyledListBoxItem.
        Dim the_item As StyledListBoxItem = _
            DirectCast(MyBase.Items(e.Index), StyledListBoxItem)
        ' Debug.WriteLine("Measure: " & the_item.Text)

        ' See how much room it will need.
        Dim hgt As Integer = 0
        Dim wid As Integer = 0

        ' Allow room for the image if present.
        If the_item.Image IsNot Nothing Then
            wid = the_item.Image.Width + 2 * ITEM_MARGIN
            hgt = the_item.Image.Height + 2 * ITEM_MARGIN
        End If

        ' Measure the text.
        If (the_item.Text IsNot Nothing) AndAlso (the_item.Text.Length > 0) Then
            Dim the_font As Font = the_item.Font
            If the_font Is Nothing Then the_font = Me.Parent.Font
            Dim item_size As SizeF = e.Graphics.MeasureString(the_item.Text, the_font)

            Dim txt_hgt As Integer = CInt(item_size.Height + 2 * ITEM_MARGIN)
            If txt_hgt > hgt Then hgt = txt_hgt

            wid += CInt(item_size.Width + 2 * ITEM_MARGIN)
        End If

        e.ItemWidth = wid
        e.ItemHeight = hgt
    End Sub

    ' Draw the item.
    Private Sub StyledListBox_DrawItem(ByVal sender As Object, ByVal e As System.Windows.Forms.DrawItemEventArgs) Handles Me.DrawItem
        If e.Index < 0 Or e.Index >= MyBase.Items.Count Then Exit Sub

        ' Convert the item into a StyledListBoxItem.
        Dim the_item As StyledListBoxItem = _
            DirectCast(MyBase.Items(e.Index), StyledListBoxItem)
        ' Debug.WriteLine("Draw: " & the_item.Text)

        ' Clear the item's drawing area.
        e.DrawBackground()
        If Me.SelectedIndex = e.Index Then e.DrawFocusRectangle()

        ' Draw the image if present.
        Dim x As Integer = ITEM_MARGIN
        If the_item.Image IsNot Nothing Then
            Dim y As Integer = _
                e.Bounds.Top + (e.Bounds.Height - the_item.Image.Height) \ 2

            e.Graphics.DrawImage(the_item.Image, x, y)
            x = the_item.Image.Width + 2 * ITEM_MARGIN
        End If

        ' Draw the text if present.
        If (the_item.Text IsNot Nothing) AndAlso (the_item.Text.Length > 0) Then
            Dim the_font As Font = the_item.Font
            If the_font Is Nothing Then the_font = Me.Parent.Font
            Dim item_size As SizeF = e.Graphics.MeasureString(the_item.Text, the_font)

            Dim y As Integer = _
                e.Bounds.Top + CInt((e.Bounds.Height - item_size.Height) / 2)

            Using the_brush As New SolidBrush(the_item.Color)
                e.Graphics.DrawString(the_item.Text, the_font, the_brush, x, y)
            End Using
        End If
    End Sub

End Class

<Serializable()> _
Public Class StyledListBoxItem
    ' The Color property.
    Private m_Color As Color
    Public Property Color() As Color
        Get
            Return m_Color
        End Get
        Set(ByVal value As Color)
            m_Color = value
        End Set
    End Property

    ' The Text property.
    Private m_Text As String
    Public Property Text() As String
        Get
            Return m_Text
        End Get
        Set(ByVal value As String)
            m_Text = value
        End Set
    End Property

    ' The Font property.
    Private m_Font As Font
    Public Property Font() As Font
        Get
            Return m_Font
        End Get
        Set(ByVal value As Font)
            m_Font = value
        End Set
    End Property

    ' The Image property.
    Private m_Image As Image
    Public Property Image() As Image
        Get
            Return m_Image
        End Get
        Set(ByVal value As Image)
            m_Image = value
        End Set
    End Property

    ' Display the object's text.
    Public Overrides Function ToString() As String
        Return Me.Text
    End Function
End Class