' This is an editor for a PolyPolyline property.
' It displays a ScribbleControl in a modal dialog.
Imports System.Drawing.Design

Public Class PolyPolylineEditor
    Inherits System.Drawing.Design.UITypeEditor

#Region "Value Editing Code"
    ' Indicates that this editor displays a modal dialog.
    Public Overloads Overrides Function GetEditStyle(ByVal context As System.ComponentModel.ITypeDescriptorContext) As System.Drawing.Design.UITypeEditorEditStyle
        Return UITypeEditorEditStyle.Modal
    End Function

    ' Edit a PolyPolyline.
    Public Overloads Overrides Function EditValue(ByVal context As System.ComponentModel.ITypeDescriptorContext, ByVal provider As System.IServiceProvider, ByVal value As Object) As Object
        ' Get an IWindowsFormsEditorService.
        Dim editor_service As IWindowsFormsEditorService = _
            CType(provider.GetService(GetType(IWindowsFormsEditorService)), _
                IWindowsFormsEditorService)

        ' If we failed to get the editor service, return the value unchanged.
        If editor_service Is Nothing Then Return value

        ' Convert the generic value into a PolyPolyline.
        Dim ppline As PolyPolyline = DirectCast(value, PolyPolyline)

        ' Make the editing dialog.
        Dim editor_dialog As New PolyPolylineEditorDialog()
        editor_dialog.PolyPolyline = ppline
        editor_dialog.EditorService = editor_service

        ' Display the editing dialog.
        editor_service.ShowDialog(editor_dialog)

        ' Save the new results.
        If editor_dialog.DialogResult = DialogResult.OK Then
            ' Return the new PolyPolyline.
            Return editor_dialog.PolyPolyline
        Else
            ' Return the old PolyPolyline.
            Return ppline
        End If
    End Function
#End Region ' Value Editing Code

#Region "Property Display Code"
    ' Indicate that we draw a representation for the Properties window's value.
    Public Overrides Function GetPaintValueSupported(ByVal context As System.ComponentModel.ITypeDescriptorContext) As Boolean
        Return True
    End Function

    ' Draw a representation for the Properties window's value
    ' by converting the scribble into a tiny image.
    Public Overrides Sub PaintValue(ByVal e As System.Drawing.Design.PaintValueEventArgs)
        ' Get the points.
        Dim ppline As PolyPolyline = DirectCast(e.Value, PolyPolyline)
        If (ppline IsNot Nothing) Then
            ' Find the points' bounds.
            Dim ppline_rect As Rectangle = ppline.Bounds()
            ppline_rect.Inflate(CInt(ppline_rect.Width * 0.1), CInt(ppline_rect.Height * 0.1))

            ' Make a bitmap to hold a small image of the scribble.
            Dim pts_wid As Integer = ppline_rect.Width
            Dim pts_hgt As Integer = ppline_rect.Height
            Dim bm_wid As Integer = e.Bounds.Width - 2
            Dim bm_hgt As Integer = e.Bounds.Height - 2
            Dim x_scale As Double = bm_wid / pts_wid
            Dim y_scale As Double = bm_hgt / pts_hgt
            Dim scale As Single = CSng(Math.Min(x_scale, y_scale))

            Dim bm As New Bitmap(bm_wid, bm_hgt)
            Using gr As Graphics = Graphics.FromImage(bm)
                Dim x0 As Integer = CInt(ppline_rect.X + (ppline_rect.Width - bm_wid / scale) / 2)
                Dim y0 As Integer = CInt(ppline_rect.Y + (ppline_rect.Height - bm_hgt / scale) / 2)
                gr.TranslateTransform(-x0, -y0)
                gr.ScaleTransform(scale, scale, Drawing2D.MatrixOrder.Append)
                gr.SmoothingMode = Drawing2D.SmoothingMode.AntiAlias
                ppline.Draw(gr, Pens.Black)
            End Using

            ' Copy the little bitmap onto the drawing surface.
            e.Graphics.InterpolationMode = Drawing2D.InterpolationMode.High
            e.Graphics.DrawImage(bm, e.Bounds.X + 1, e.Bounds.Y + 1)
        End If
    End Sub
#End Region ' Property Display Code

End Class
