Imports System.Xml

Public Class Form1
    ' Let the user select an XML file.
    Private Sub btnSelectFile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSelectFile.Click
        If dlgXmlFile.ShowDialog() = Windows.Forms.DialogResult.OK Then
            txtFile.Text = dlgXmlFile.FileName
        End If
    End Sub

    ' Let the user select an output directory.
    Private Sub btnSelectDirectory_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSelectDirectory.Click
        dlgOutputDir.SelectedPath = txtDirectory.Text
        If dlgOutputDir.ShowDialog() = Windows.Forms.DialogResult.OK Then
            txtDirectory.Text = dlgOutputDir.SelectedPath
        End If
    End Sub

    Private Sub btnGo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGo.Click
        ' Load and parse the XML file.
        Dim root_node As DocNode = ParseXmlDocument(txtFile.Text)

        ' Sort the nodes' children.
        root_node.SortChildren()

        ' Generate results.
        Dim output_dir As String = txtDirectory.Text
        If Not output_dir.EndsWith("\") Then output_dir &= "\"

        Dim assembly_title As String = "Assembly " & root_node.ShortName

        ' Make the index page.
        MakeIndex(root_node, output_dir, assembly_title)

        ' Make pages for the individual objects.
        MakeObjectPages(root_node, output_dir)

        MessageBox.Show("Done")
    End Sub

    ' Parse the XML document.
    Private Function ParseXmlDocument(ByVal file_name As String) As DocNode
        ' Load the XML file.
        Dim xml_doc As New XmlDocument()
        xml_doc.Load(file_name)

        ' Find the <assembly> node.
        Dim assembly_node As XmlNode = xml_doc.DocumentElement.Item("assembly")

        ' Get the <name> value.
        Dim root As New DocNode(assembly_node, assembly_node.Item("name").InnerText.Trim, "A")

        ' Add the <members> to the root node.
        Dim members_node As XmlNode = xml_doc.DocumentElement.Item("members")
        For Each member As XmlNode In members_node.ChildNodes
            ' Make the child node.
            Dim full_name As String = member.Attributes("name").Value
            Debug.Assert(full_name.Contains(":"), "Member node's name does not contain ':'")
            Dim type_code As String = full_name.Substring(0, full_name.IndexOf(":"))
            full_name = full_name.Substring(full_name.IndexOf(":") + 1)
            Dim child_node As New DocNode(member, full_name, type_code)

            ' Add the child to the root node.
            root.AddChild(child_node)
        Next member

        Return root
    End Function

    ' Make the index page.
    Private Sub MakeIndex(ByVal root_node As DocNode, ByVal output_dir As String, ByVal assembly_title As String)
        ' Make the file's title.
        Dim txt As String = My.Resources.IndexTemplate
        txt = txt.Replace("@TITLE@", assembly_title & ": Index")

        ' Make the file's body.
        Dim lines As String = root_node.PrefixOrder(True, False, False, False, True)
        lines = lines.Replace(vbTab, "&nbsp;&nbsp;&nbsp;&nbsp;")
        txt = txt.Replace("@BODY@", lines)

        ' Write the file.
        My.Computer.FileSystem.WriteAllText(output_dir & "index.html", txt, False)
    End Sub

    ' Make the object pages.
    Private Sub MakeObjectPages(ByVal root_node As DocNode, ByVal output_dir As String)
        root_node.MakeWebPages(output_dir)
    End Sub

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        txtFile.Text = My.Settings.Item("InputXml").ToString()
        txtDirectory.Text = My.Settings.Item("OutputDir").ToString()
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        My.Settings.Item("InputXml") = txtFile.Text
        My.Settings.Item("OutputDir") = txtDirectory.Text
        My.Settings.Save()
    End Sub
End Class
