''' <summary>
''' Represents an employee.
''' </summary>
''' <remarks>Inherits from Person.</remarks>
''' <SeeAlsoSection>
'''   <seealso cref="Person">Person parent class</seealso>
''' </SeeAlsoSection>
''' <scope>Public</scope>
''' <History>
'''   <Change By="Rod Stephens" Date="4/1/2007">Initial creation.</Change>
'''   <Change By="Rod Stephens" Date="4/3/2007">Added comments.</Change>
'''   <Change By="Zaphod" Date="4/13/2007">Deleted confusing stuff.</Change>
''' </History>
Public Class Employee
    Inherits Person

    ''' <summary>
    ''' Types of employees.
    ''' </summary>
    ''' <remarks>
    ''' Indicates the type of employee.
    ''' <list type="table">
    ''' <listheader><term>Member Name</term><description>Description</description></listheader>
    ''' <item>
    '''   <term><see cref="EmployeeClasses.Developer">Developer</see>
    '''     <seealso cref="EmployeeClasses.Developer">The Developer entry</seealso>
    '''   </term>
    '''   <description>A front-line developer.</description>
    ''' </item>
    ''' <item>
    '''   <term><see cref="EmployeeClasses.ProjectManager">ProjectManager</see>
    '''     <seealso cref="EmployeeClasses.ProjectManager">Field ProjectManager</seealso>
    '''   </term>
    '''   <description>A project manager.</description>
    ''' </item>
    ''' <item>
    '''   <term><see cref="EmployeeClasses.VicePresident">VicePresident</see>
    '''     <seealso cref="EmployeeClasses.VicePresident">Vice President</seealso>
    '''   </term>
    '''   <description>Does executive stuff.</description>
    ''' </item>
    ''' </list>
    ''' Also look at the <see cref="XmlTestClasses.Employee">Employee</see> class.
    ''' </remarks>
    ''' <scope>Public</scope>
    ''' <History>
    ''' <Change By="Rod Stephens" Date="4/2/2007">Initial creation.</Change>
    ''' <Change By="Rod Stephens" Date="4/2/2007">Added VicePresident.</Change>
    ''' </History>
    Public Enum EmployeeClasses
        ''' <summary>A front-line developer.</summary>
        ''' <remarks>Programmer.</remarks>
        Developer

        ''' <summary>A project manager.</summary>
        ''' <remarks>Manages development projects.</remarks>
        ProjectManager

        ''' <summary>Executive.</summary>
        ''' <remarks>Does executive stuff.</remarks>
        VicePresident
    End Enum

    ''' <summary>
    ''' The private variable that holds the EmployeeClass value.
    ''' </summary>
    ''' <remarks>Use the property procedures to get and set this value.</remarks>
    ''' <scope>Private</scope>
    Private m_EmployeeClass As String

    ''' <summary>
    ''' The Employees's type.
    ''' </summary>
    ''' <value>The Employee's class.</value>
    ''' <returns>An employee's EmployeeClass.</returns>
    ''' <remarks>Used to determine responsibilities.</remarks>
    ''' <scope>Public</scope>
    ''' <SeeAlsoSection>
    '''   <seealso cref="EmployeeClasses">EmployeeClasses</seealso>
    ''' </SeeAlsoSection>
    Public Property EmployeeClass() As String
        Get
            Return m_EmployeeClass
        End Get
        Set(ByVal value As String)
            m_EmployeeClass = value
        End Set
    End Property

    ''' <summary>
    ''' The private variable that holds the Office value.
    ''' </summary>
    ''' <remarks>Use the property procedures to get and set this value.</remarks>
    ''' <scope>Private</scope>
    Private m_Office As String

    ''' <summary>
    ''' The Employees's building and office number.
    ''' </summary>
    ''' <value>The Employee's office number.</value>
    ''' <returns>An employee's office number.</returns>
    ''' <remarks>Used in mailing addresses.</remarks>
    ''' <scope>Public</scope>
    Public Property Office() As String
        Get
            Return m_Office
        End Get
        Set(ByVal value As String)
            m_Office = value
        End Set
    End Property
End Class
