Imports NUnit.Framework ' C:\Program Files\NUnit-Net-2.0 2.2.8\bin\nunit.framework.dll
Imports System.CodeDom.Compiler
Imports System.Reflection
Imports System.Math

<TestFixture()> Public Class ExpressionLibTester
    Private m_Variables As Dictionary(Of String, Double)
    Private m_Random As New Random

    ' Create a new CompiledExpressionTest.
    Public Sub New()
        MyBase.New()
    End Sub

    ' Initialize. Prepare to run the tests.
    <TestFixtureSetUp()> Public Sub Init()
        ' Make some parameters.
        m_Variables = New Dictionary(Of String, Double)
        m_Variables.Add("A", 123.456)
        m_Variables.Add("B", 7463.218)
        m_Variables.Add("C", 0.000000367)
        m_Variables.Add("D", 2.235E+210)
    End Sub

    ' Run the basic tests.
    <Test()> Public Sub BasicTests()
        RunTest(".1", m_Variables)
        RunTest("0.1", m_Variables)
        RunTest("1234", m_Variables)
        RunTest("A", m_Variables)
        RunTest("(17)", m_Variables)
        RunTest("-13", m_Variables)
        RunTest("3 * -2", m_Variables)
        RunTest("3*-2", m_Variables)
        RunTest("+37", m_Variables)
        RunTest("2 * +3", m_Variables)
        RunTest("2*+3", m_Variables)
        RunTest("3^2", m_Variables)
        RunTest("6*7", m_Variables)
        RunTest("4/5", m_Variables)
        RunTest("4\5", m_Variables)
        RunTest("7%4", m_Variables)
        RunTest("3+9", m_Variables)
        RunTest("3-9", m_Variables)
        RunTest("Sin(A)", m_Variables)
        RunTest("Cos(B)", m_Variables)
        RunTest("Tan(C)", m_Variables)
        RunTest("Sqrt(D)", m_Variables)
        RunTest("Log10(1010101)", m_Variables)
        RunTest("Log(1010101)", m_Variables)
        RunTest("Exp(13.37)", m_Variables)
        RunTest("ATan(1)", m_Variables)
        RunTest("ATan(-21)", m_Variables)
        RunTest("ATan(12)", m_Variables)
        RunTest("ASin(.5)", m_Variables)
        RunTest("ACos(.5)", m_Variables)
        RunTest("ASin(0.5)", m_Variables)
        RunTest("ACos(0.5)", m_Variables)
    End Sub

    ' Run some misc tests.
    <Test()> Public Sub MiscTests()
        RunTest("1+2", m_Variables)
        RunTest("1+2+3", m_Variables)
        RunTest("2+3*4", m_Variables)
        RunTest("2+(3*4)", m_Variables)
        RunTest("2*3+4", m_Variables)
        RunTest("2*(3+4)", m_Variables)
        RunTest("13^7-12/19*25", m_Variables)
        RunTest("563 % 12", m_Variables)
        RunTest("2*3+4", m_Variables)
        RunTest("2*(3+4)", m_Variables)
        RunTest("2*3+4", m_Variables)
        RunTest("2*(3+4)", m_Variables)
        RunTest("A*B-C/D*A", m_Variables)
    End Sub

    ' This test fails.
    <Test()> Public Sub TestFails()
        RunTest("10 Mod 3", m_Variables)
    End Sub

    ' Added to catch bug in Tan(-(A)^-ATan(D)).
    <Test()> Public Sub BugFixTests()
        RunTest("-1^-2", m_Variables)
        RunTest("(-123.456)^(-1.5707963267949)", m_Variables)
        RunTest("-(A)^-ATan(D)", m_Variables)
        RunTest("-(A)", m_Variables)
        RunTest("-ATan(D)", m_Variables)
        RunTest("Tan(-(A)^-ATan(D))", m_Variables)
    End Sub

    ' Weird cases.
    <Test()> Public Sub WeirdTests()
        RunTest("2\0", m_Variables)
        RunTest("2/0", m_Variables)
        RunTest("1^0", m_Variables)
        RunTest("1^-1", m_Variables)
        RunTest("70.7^-12.3", m_Variables)
        RunTest("Sqrt(0)", m_Variables)
        RunTest("Sqrt(-1)", m_Variables)
        RunTest("Exp(1337)", m_Variables)
        RunTest("ATan(0)", m_Variables)
        RunTest("ASin(12)", m_Variables)
        RunTest("ACos(12)", m_Variables)
        RunTest("", m_Variables)
        RunTest("()", m_Variables)
        RunTest("(-1)", m_Variables)
        RunTest("(((-1)))", m_Variables)
        RunTest("(-1", m_Variables)
        RunTest("(-1))", m_Variables)
        RunTest("-1)", m_Variables)
        RunTest("((-1)", m_Variables)
        RunTest("()-1", m_Variables)
    End Sub

    ' Perform some random tests.
    <Test()> Public Sub RandomTests()
        Const NUM_TESTS As Integer = 10
        Const MAX_EXPRESSION_DEPTH As Integer = 5
        For i As Integer = 1 To NUM_TESTS
            ' Make a random test.
            Dim expression_string As String = RandomExpression(m_Random.Next(1, MAX_EXPRESSION_DEPTH))
            RunTest(expression_string, m_Variables)
        Next i
    End Sub

    ' This test looks for an exception.
    <Test(), ExpectedException(GetType(InvalidCastException))> Public Sub ExpectAnException()
        Throw New InvalidCastException()
    End Sub

#Region "Helper Routines"
    ' Evaluate the expression using a CompiledExpression
    ' and compiled VB code and compare the results.
    ' Throw an error if the values are different.
    Private Sub RunTest(ByVal expression_string As String, ByVal variables As Dictionary(Of String, Double))
        Dim result1 As Double
        Dim result2 As Double
        Dim err1 As Boolean = False
        Dim err2 As Boolean = False

        ' Calculate the result using a CompiledExpression.
        Try
            Dim expr As New CompiledExpression(expression_string)
            result1 = expr.Evaluate(variables)
        Catch ex As Exception
            err1 = True
            Debug.WriteLine( _
                "*** CompiledExpression Error ***" & _
                vbCrLf & ex.Message & vbCrLf)
        End Try

        ' Calculate the result using compiled VB code.
        Try
            result2 = VbResult(expression_string.Replace("%", " Mod "), variables)
        Catch ex As Exception
            err2 = True
            Debug.WriteLine( _
                "*** CompiledExpression VB ***" & _
                vbCrLf & ex.Message & vbCrLf)
        End Try

        ' Compare the results.
        Dim diff As Double = result2 - result1

        ' Pass/Fail.
        If (err1 AndAlso err2) OrElse _
            (Double.IsNaN(result1) AndAlso Double.IsNaN(result2)) OrElse _
            (Double.IsInfinity(result1) AndAlso Double.IsInfinity(result2)) OrElse _
            (Abs(diff) < 0.000000001) _
        Then
            ' Passed.
            Exit Sub
        Else
            ' Failed.
            Throw New Exception("Expression failed: " & expression_string)
        End If
    End Sub

    ' Compile some VB code and use it to perform the calculation.
    Private Function VbResult(ByVal expression_string As String, ByVal variables As Dictionary(Of String, Double)) As Double
        ' Compose the function's parameter list.
        Dim params As String = ""
        Dim param_name As String = "A"
        For i As Integer = 0 To variables.Count - 1
            params &= ", ByVal " & param_name & " As Double"
            param_name = Chr(Asc(param_name) + 1)
        Next i
        If params.Length > 0 Then params = params.Substring(2)

        ' Compose the code.
        Dim vb_code As String = _
            "Imports System" & vbCrLf & _
            "Imports System.Math" & vbCrLf & _
            "Namespace MainNamespace" & vbCrLf & _
            "  Class MainClass" & vbCrLf & _
            "    Public Function Main(" & params & ") As Double" & vbCrLf & _
            "        Return " & expression_string & vbCrLf & _
            "    End Function" & vbCrLf & _
            "  End Class" & vbCrLf & _
            "End Namespace"

        ' Prepare the compiler parameters.
        ' Add any needed references.
        Dim compiler_params As CompilerParameters = New CompilerParameters
        compiler_params.ReferencedAssemblies.Add("system.dll")

        ' Generate a library in memory.
        compiler_params.CompilerOptions = "/t:library"
        compiler_params.GenerateInMemory = True

        ' Compile the code.
        Dim code_provider As VBCodeProvider = New VBCodeProvider
        Dim compiler_results As CompilerResults = _
            code_provider.CompileAssemblyFromSource(compiler_params, vb_code)
        code_provider.Dispose()

        ' Check for errors.
        ' CompilerErrors = compiler_results.Errors
        If compiler_results.Errors.Count > 0 Then
            ' There were errors.
            Dim txt As String = "Compiler error:" & vbCrLf & vbCrLf
            For Each compiler_error As CompilerError In compiler_results.Errors
                txt &= vbTab & compiler_error.ErrorText & vbCrLf
            Next compiler_error
            Throw New Exception(txt)
        Else
            ' There were no errors. Try to execute the code.
            ' Get the assembly.
            Dim the_assembly As System.Reflection.Assembly = _
                compiler_results.CompiledAssembly

            ' Make a MainClass object.
            Dim main_object As Object = _
                the_assembly.CreateInstance("MainNamespace.MainClass")

            ' Get the MainClass's type.
            Dim main_object_type As Type = main_object.GetType

            ' Get the MainClass's Main method.
            Dim run_method_info As MethodInfo
            run_method_info = main_object_type.GetMethod("Main")

            ' Make the parameters array.
            Dim parameters(0 To variables.Count - 1) As Object
            Dim vals As Dictionary(Of String, Double).ValueCollection
            vals = variables.Values
            Dim param_num As Integer = 0
            For Each val As Double In vals
                parameters(param_num) = val
                param_num += 1
            Next val

            ' Evaluate the expression.
            Dim result As Double
            Try
                result = CDbl(run_method_info.Invoke(main_object, parameters))
            Catch ex As Exception
                ' Rethrow the exception.
                Throw ex
            End Try
            Return result
        End If
    End Function

    ' Recursively make a random expression.
    Private Function RandomExpression(ByVal depth As Integer) As String
        If depth < 1 Then
            ' Use a constant or variable.
            Select Case m_Random.Next(8)
                Case 0 To 3
                    Return (m_Random.NextDouble * 10000).ToString()
                Case 4
                    Return "A"
                Case 5
                    Return "B"
                Case 6
                    Return "C"
                Case 7
                    Return "D"
            End Select
        Else
            ' Pick an operator.
            Select Case m_Random.Next(20)
                Case 0  ' Parens.
                    Return "(" & RandomExpression(depth - 1) & ")"
                Case 1  ' Unary minus.
                    Return "-" & RandomExpression(depth - 1)
                Case 2  ' Unary plus.
                    Return "+" & RandomExpression(depth - 1)
                Case 3  ' ^
                    Return RandomExpression(depth - 1) & " ^ " & RandomExpression(depth - 1)
                Case 4  ' *
                    Return RandomExpression(depth - 1) & " * " & RandomExpression(depth - 1)
                Case 5  ' /
                    Return RandomExpression(depth - 1) & " / " & RandomExpression(depth - 1)
                Case 6  ' \
                    Return RandomExpression(depth - 1) & " \ " & RandomExpression(depth - 1)
                Case 7  ' %
                    Return RandomExpression(depth - 1) & " % " & RandomExpression(depth - 1)
                Case 8  ' +
                    Return RandomExpression(depth - 1) & " + " & RandomExpression(depth - 1)
                Case 9  ' -
                    Return RandomExpression(depth - 1) & " - " & RandomExpression(depth - 1)
                Case 10 ' Sin
                    Return "Sin(" & RandomExpression(depth - 1) & ")"
                Case 11 ' Cos
                    Return "Cos(" & RandomExpression(depth - 1) & ")"
                Case 12 ' Tan
                    Return "Tan(" & RandomExpression(depth - 1) & ")"
                Case 13 ' Sqrt
                    Return "Sqrt(" & RandomExpression(depth - 1) & ")"
                Case 14 ' Log10
                    Return "Log10(" & RandomExpression(depth - 1) & ")"
                Case 15 ' Log
                    Return "Log(" & RandomExpression(depth - 1) & ")"
                Case 16 ' Exp
                    Return "Exp(" & RandomExpression(depth - 1) & ")"
                Case 17 ' ATan
                    Return "ATan(" & RandomExpression(depth - 1) & ")"
                Case 18 ' ASin
                    Return "ASin(" & RandomExpression(depth - 1) & ")"
                Case 19 ' ACos
                    Return "ACos(" & RandomExpression(depth - 1) & ")"
            End Select
        End If

        Return Nothing
    End Function
#End Region ' Helper Routines

End Class
