Imports System.Drawing.Drawing2D

' At design time, set:
'       Form.FormBorderStyle = None
'       Form.ControlBox = False
'       Form.TopMost = True
'       lblTitle.Font.Unit = Pixel.
'       Label BcakColors = Transparent
'       txtSerialNumber.ReadOnly = True

Public Class frmSplash
    ' Set the progress.
    Public Delegate Sub ShowProgressDelegate(ByVal progress As Integer)
    Public Sub ShowProgress(ByVal progress As Integer)
        pbarLoad.Value = progress
    End Sub

    ' Get or set the progress indicator.
    Public Property Progress() As Integer
        Get
            Return pbarLoad.Value
        End Get
        Set(ByVal value As Integer)
            If Me.InvokeRequired Then
                Dim show_progress_delegate As ShowProgressDelegate
                show_progress_delegate = AddressOf ShowProgress
                Me.Invoke(show_progress_delegate, New Object() {value})
            Else
                ShowProgress(value)
            End If
        End Set
    End Property

    ' Prepare the splash screen.
    Private Sub frmSplash_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Shape the form.
        ' Make a bitmap for our form image.
        Dim bm As New Bitmap(Me.ClientSize.Width, Me.ClientSize.Height)
        Dim gr As Graphics = Graphics.FromImage(bm)

        ' Draw smooth curves.
        gr.SmoothingMode = SmoothingMode.AntiAlias

        ' Make a rounded rectangle for the text area.
        Using rect_path As GraphicsPath = RoundedRectanglePath(panTextArea.Bounds(), 30, 30)
            ' Draw the text area.
            Using br As New SolidBrush(panTextArea.BackColor)
                gr.FillPath(br, rect_path)
            End Using
            Using thick_pen As New Pen(Color.DarkBlue, 3)
                gr.DrawPath(thick_pen, rect_path)
            End Using
        End Using

        ' Make a Bitmap containing the Jack image.
        Dim jack_bm As New Bitmap(My.Resources.Jack)
        jack_bm.MakeTransparent(bm.GetPixel(0, 0))

        ' Draw the Jack image onto the form's Bitmap.
        gr.DrawImage(jack_bm, jack_bm.GetBounds(GraphicsUnit.Pixel))

        ' Move controls out of panTextArea.
        Dim ctls(panTextArea.Controls.Count - 1) As Control
        panTextArea.Controls.CopyTo(ctls, 0)
        For Each ctl As Control In ctls
            ctl.Parent = Me
            ctl.Left += panTextArea.Left
            ctl.Top += panTextArea.Top
        Next ctl

        ' Draw "Jack!"
        Using jack_path As New GraphicsPath()
            ' Make a path for the text.
            Using string_format As New StringFormat
                jack_path.AddString( _
                    lblTitle.Text, _
                    lblTitle.Font.FontFamily, _
                    lblTitle.Font.Style, _
                    lblTitle.Font.Size, _
                    lblTitle.Bounds(), _
                    string_format)
            End Using

            ' Fill the text with a gradient.
            Using jack_br As New LinearGradientBrush( _
              jack_path.GetBounds(), Color.Red, Color.Lime, LinearGradientMode.Horizontal)
                Dim color_blend As New ColorBlend()
                color_blend.Colors = New Color() {Color.Red, Color.Yellow, Color.Lime}
                color_blend.Positions = New Single() {0.0, 0.5, 1.0}
                jack_br.InterpolationColors = color_blend

                gr.FillPath(jack_br, jack_path)
            End Using

            Using jack_pen As New Pen(Color.DarkBlue, 2)
                gr.DrawPath(jack_pen, jack_path)
            End Using
        End Using

        ' Draw the URL rotated over lblUrl.
        If lblUrl.Visible Then
            Const URL_TEXT As String = "www.vb-helper.com"
            gr.RotateTransform(90, MatrixOrder.Append)
            Dim url_size As SizeF = gr.MeasureString(URL_TEXT, lblUrl.Font, 1000)
            gr.TranslateTransform(lblUrl.Left + url_size.Height, lblUrl.Top, MatrixOrder.Append)
            gr.DrawString(URL_TEXT, lblUrl.Font, Brushes.Blue, 0, 0)

            ' Move lblUrl under the text so we can catch its Click event.
            lblUrl.Width = CInt(url_size.Height)
            lblUrl.Height = CInt(url_size.Width)
            lblUrl.Text = ""
        End If

        ' Make black areas transparent.
        Me.BackColor = Color.Black
        Me.TransparencyKey = Me.BackColor

        ' Use the result for the form's background image.
        Me.BackgroundImage = bm

        ' Display the version and copyright information.
        lblVersion.Text = "Version " & My.Application.Info.Version.ToString()
        txtSerialNumber.Text = "35238-ARGH-R17"
        lblCopyright.Text = My.Application.Info.Copyright
    End Sub

    ' Display as a splash screen.
    Public Sub ShowSplash(ByVal owner As IWin32Window)
        Me.Show(owner)
    End Sub

    ' Display as an About dialog.
    Public Sub ShowAbout(ByVal owner As IWin32Window)
        btnOk.Visible = True
        lblSurprise.Visible = True
        pbarLoad.Visible = False
        lblUrl.Visible = True

        Me.ShowDialog(owner)
    End Sub

    ' Close when shown as an About dialog.
    Private Sub btnOk_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOk.Click
        Me.Close()
    End Sub

    ' Display a surprise form.
    Private Sub lblSurprise_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles lblSurprise.Click
        Dim frm As New frmSurprise
        frm.ShowDialog(Me)
    End Sub

    ' Open the VB Helper Web site.
    Private Sub lblUrl_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles lblUrl.Click
        System.Diagnostics.Process.Start("http://www.vb-helper.com")
    End Sub
End Class