Imports System.ComponentModel
Imports System.Threading
Imports System.Math

Public Class Form1
    ' The graphing PictureBoxes.
    Private m_Graphs() As PictureBox

    Private m_Wid As Integer
    Private m_Hgt As Integer

    ' The graphs' images.
    Private m_Bitmaps() As Bitmap
    Private m_LastY() As Integer

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Save the PictureBoxes in an array.
        m_Graphs = New PictureBox() {picGraph0, picGraph1, picGraph2, picGraph3}

        ' Get the PictureBoxes' dimensions.
        m_Wid = picGraph0.ClientSize.Width
        m_Hgt = picGraph0.ClientSize.Height

        ' Make the bitmaps.
        Dim num_graphs As Integer = m_Graphs.Length
        ReDim m_Bitmaps(0 To num_graphs - 1)
        ReDim m_LastY(0 To num_graphs - 1)
        For i As Integer = 0 To 3
            m_Bitmaps(i) = New Bitmap(m_Wid, m_Hgt)
            Using gr As Graphics = Graphics.FromImage(m_Bitmaps(i))
                gr.Clear(Color.White)
            End Using
            m_Graphs(i).Image = m_Bitmaps(i)
        Next i

        ' Start the background workers.
        m_LastY(0) = Y0(0)
        bgwGraph0.RunWorkerAsync()

        m_LastY(1) = m_Hgt \ 2
        bgwGraph1.RunWorkerAsync()

        m_LastY(2) = m_Hgt \ 2
        bgwGraph2.RunWorkerAsync()

        m_LastY(3) = Y3(0)
        bgwGraph3.RunWorkerAsync()
    End Sub

    ' Plot a data point.
    Private Delegate Sub PlotPointDelegate(ByVal graph_num As Integer, ByVal Y As Integer)
    Private m_PlotPointDelegate As PlotPointDelegate = AddressOf Me.PlotPoint
    Private Sub PlotPoint(ByVal graph_num As Integer, ByVal Y As Integer)
        ' Scoot the image over to the left.
        Dim bm As New Bitmap(m_Wid, m_Hgt)
        Using gr As Graphics = Graphics.FromImage(bm)
            gr.DrawImage(m_Bitmaps(graph_num), -1, 0)

            ' Clear the right edge.
            gr.DrawLine(Pens.White, m_Wid - 1, 0, m_Wid - 1, m_Hgt)

            ' Draw the new point.
            gr.DrawLine(Pens.Blue, m_Wid - 2, m_LastY(graph_num), m_Wid - 1, Y)
        End Using

        ' Save the new Y value.
        m_LastY(graph_num) = Y

        ' Refresh the graph.
        m_Bitmaps(graph_num) = bm
        m_Graphs(graph_num).Image = bm
    End Sub

    ' Draw graph 0.
    Private Sub bgwGraph0_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles bgwGraph0.DoWork
        ' Work at reduced priority.
        Thread.CurrentThread.Priority = ThreadPriority.BelowNormal

        Static X As Integer = 0
        Do
            ' Calculate the next point.
            X += 1

            ' Report the point to the UI thread.
            Me.Invoke(Me.m_PlotPointDelegate, 0, Y0(X))

            ' Pause a while.
            Thread.Sleep(1)

            ' See if we should cancel.
            If Me.bgwGraph0.CancellationPending Then Exit Do
        Loop
    End Sub
    Private Function Y0(ByVal x As Integer) As Integer
        Return CInt(m_Hgt / 2 + 10 * Sin(x / 11) + 5 * Sin(x / 7))
    End Function

    ' Draw graph 1.
    Private Sub bgwGraph1_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles bgwGraph1.DoWork
        ' Work at reduced priority.
        Thread.CurrentThread.Priority = ThreadPriority.BelowNormal

        Dim min_rnd As Integer = -5
        Dim max_rnd As Integer = 5
        Dim rand As New Random
        Dim Y As Integer = m_LastY(1)
        Do
            ' Calculate the next point.
            Y += rand.Next(min_rnd, max_rnd)
            If Y < 0 Then
                min_rnd = -5
                max_rnd = 6
                Y = 0
            End If
            If Y > m_Hgt Then
                min_rnd = -6
                max_rnd = 5
                Y = m_Hgt
            End If

            ' Report the point to the UI thread.
            Me.Invoke(Me.m_PlotPointDelegate, 1, Y)

            ' Pause a while.
            Thread.Sleep(1)

            ' See if we should cancel.
            If Me.bgwGraph1.CancellationPending Then Exit Do
        Loop
    End Sub

    ' Draw graph 2.
    Private Sub bgwGraph2_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles bgwGraph2.DoWork
        ' Work at reduced priority.
        Thread.CurrentThread.Priority = ThreadPriority.BelowNormal

        Dim rand As New Random
        Dim Y As Integer = m_LastY(1)
        Do
            ' Calculate the next point.
            Y += rand.Next(-20, 20)
            If Y < 0 Then Y = 0
            If Y > m_Hgt Then Y = m_Hgt

            ' Report 20 points to the UI thread.
            For i As Integer = 1 To rand.Next(10, 40)
                Thread.Sleep(1)
                Me.Invoke(Me.m_PlotPointDelegate, 2, Y)

                ' Pause a while.
                Thread.Sleep(1)

                ' See if we should cancel.
                If Me.bgwGraph2.CancellationPending Then Exit Do
            Next i
        Loop
    End Sub

    ' Draw graph 3.
    Private Sub bgwGraph3_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles bgwGraph3.DoWork
        ' Work at reduced priority.
        Thread.CurrentThread.Priority = ThreadPriority.BelowNormal

        Static X As Integer = 0
        Do
            ' Calculate the next point.
            X += 1

            ' Report the point to the UI thread.
            Me.Invoke(Me.m_PlotPointDelegate, 3, Y3(X))

            ' Pause a while.
            Thread.Sleep(0)

            ' See if we should cancel.
            If Me.bgwGraph3.CancellationPending Then Exit Do
        Loop
    End Sub
    Private Function Y3(ByVal x As Integer) As Integer
        Return CInt(m_Hgt / 2 + 3 * Sin(0.1 + x / 3) + 5 * Sin(0.2 + x / 5) + 7 * Sin(0.3 + x / 7) + 11 * Sin(0.4 + x / 11) + 13 * Sin(0.5 + x / 13))
    End Function

    ' Stop a worker.
    Private Sub btnStop0_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStop0.Click
        ToggleWorker(btnStop0, bgwGraph0)
    End Sub
    Private Sub btnStop1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStop1.Click
        ToggleWorker(btnStop1, bgwGraph1)
    End Sub
    Private Sub btnStop2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStop2.Click
        ToggleWorker(btnStop2, bgwGraph2)
    End Sub
    Private Sub btnStop3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStop3.Click
        ToggleWorker(btnStop3, bgwGraph3)
    End Sub
    Private Sub ToggleWorker(ByVal btn As Button, ByVal bgw As BackgroundWorker)
        If bgw.IsBusy Then
            bgw.CancelAsync()
            btn.Text = "Start"
        Else
            bgw.RunWorkerAsync()
            btn.Text = "Stop"
        End If
    End Sub
End Class
