Imports System.Math
Imports System.ComponentModel
Imports System.Threading

Public Class Form1

#Region "Variables"
    ' Variables for rubberband drawing.
    Private m_DrawingBox As Boolean
    Private m_X1, m_Y1, m_X2, m_Y2 As Integer
    Private m_ZoomingBitmap As Bitmap
    Private m_ZoomingGraphics As Graphics

    ' Drawing variables.
    Private m_Bitmap As Bitmap

    ' World coordinates.
    Private Const MIN_X As Double = -2.2
    Private Const MAX_X As Double = 1
    Private Const MIN_Y As Double = -1.2
    Private Const MAX_Y As Double = 1.2
    Private m_Wxmin As Double = MIN_X
    Private m_Wxmax As Double = MAX_X
    Private m_Wymin As Double = MIN_Y
    Private m_Wymax As Double = MAX_Y

    ' Calculation values.
    Public Const MaxIterations As Integer = 128

    ' Colors.
    Public m_NumColors As Integer
    Private m_Colors() As Color

#End Region ' Variables

#Region "Form Code"

    ' Draw the first image.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Load the colors.
        m_NumColors = 15
        ReDim m_Colors(m_NumColors)
        For i As Integer = 0 To m_NumColors - 1
            m_Colors(i) = Color.FromArgb( _
                QBColor(i + 1) + &HFF000000)
        Next i

        ' Force the form to appear.
        Me.Show()

        ' Draw at the current (full) scale.
        DrawMandelbrot()
    End Sub

    ' Redraw with the new form size.
    Private Sub Form1_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        ' Do nothing if background workers are running.
        If m_NumWorkersRunning > 0 Then Exit Sub

        ' Do nothing if we haven't loaded yet.
        If m_Colors Is Nothing Then Exit Sub

        ' If the form is minimized, do nothing.
        If Me.WindowState = FormWindowState.Minimized Then Exit Sub

        ' See if we just moved and didn't resize.
        If m_Bitmap IsNot Nothing Then
            ' We have a Bitmap. 
            ' If it's already the right size, do nothing.
            If m_Bitmap.Width = picCanvas.ClientSize.Width And _
               m_Bitmap.Height = picCanvas.ClientSize.Height _
                Then Exit Sub
        End If

        ' Draw at the current scale.
        DrawMandelbrot()
    End Sub

    ' Return to full scale.
    Private Sub btnFullScale_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnFullScale.Click
        m_Wxmin = MIN_X
        m_Wxmax = MAX_X
        m_Wymin = MIN_Y
        m_Wymax = MAX_Y

        DrawMandelbrot()
    End Sub

    ' Stop drawing.
    Private Sub btnStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStop.Click
        btnStop.Enabled = False

        ' Stop the workers.
        For i As Integer = 0 To NUM_WORKERS - 1
            If m_Workers(i).IsBusy Then m_Workers(i).CancelAsync()
        Next i
    End Sub

#End Region ' Form Code

#Region "Rubberbanding"
    ' Start rubberband drawing.
    Private Sub picCanvas_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseDown
        ' Ignore mouse events if a BackgroundWorker is running.
        If m_NumWorkersRunning > 0 Then Exit Sub

        m_DrawingBox = True
        m_X1 = e.X
        m_Y1 = e.Y
        m_X2 = m_X1
        m_Y2 = m_Y1

        ' Make a copy of the current bitmap.
        m_ZoomingBitmap = New Bitmap(m_Bitmap)
        m_ZoomingGraphics = Graphics.FromImage(m_ZoomingBitmap)
    End Sub

    ' Continue rubberband drawing.
    Private Sub picCanvas_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseMove
        If Not m_DrawingBox Then Exit Sub

        ' Save the new corner.
        m_X2 = e.X
        m_Y2 = e.Y

        ' Draw the new box.
        Dim gr As Graphics = picCanvas.CreateGraphics()
        Dim rect As New Rectangle
        rect.X = Min(m_X1, m_X2)
        rect.Y = Min(m_Y1, m_Y2)
        rect.Width = Abs(m_X2 - m_X1)
        rect.Height = Abs(m_Y2 - m_Y1)
        m_ZoomingGraphics.DrawImage(m_Bitmap, 0, 0)
        m_ZoomingGraphics.DrawRectangle(Pens.White, rect)

        ' Display the result.
        picCanvas.Image = m_ZoomingBitmap
    End Sub

    ' Finish rubberband drawing.
    Private Sub picCanvas_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseUp
        If Not m_DrawingBox Then Exit Sub
        m_DrawingBox = False

        ' Dispose of the zooming Bitmap and Graphics.
        picCanvas.Image = Nothing
        m_ZoomingBitmap.Dispose()
        m_ZoomingGraphics.Dispose()
        m_ZoomingBitmap = Nothing
        m_ZoomingGraphics = Nothing

        ' Save the new corner.
        m_X2 = e.X
        m_Y2 = e.Y

        ' Put the selected coordinates in order.
        Dim x1, x2, y1, y2, factor As Double
        x1 = Min(m_X1, m_X2)
        x2 = Max(m_X1, m_X2)
        y1 = Min(m_Y1, m_Y2)
        y2 = Max(m_Y1, m_Y2)

        ' Convert screen coords into drawing coords.
        factor = (m_Wxmax - m_Wxmin) / picCanvas.ClientSize.Width
        m_Wxmax = m_Wxmin + x2 * factor
        m_Wxmin = m_Wxmin + x1 * factor

        factor = (m_Wymax - m_Wymin) / picCanvas.ClientSize.Height
        m_Wymax = m_Wymin + y2 * factor
        m_Wymin = m_Wymin + y1 * factor

        ' Draw the new Mandelbrot set.
        DrawMandelbrot()
    End Sub
#End Region ' Rubberbanding

#Region "Aspect Code"

    ' Adjust the aspect ratio of the selected
    ' coordinates so they fit the window properly.
    Private Sub AdjustAspect()
        Dim want_aspect, picCanvas_aspect As Double
        Dim hgt, wid, mid As Double

        want_aspect = (m_Wymax - m_Wymin) / (m_Wxmax - m_Wxmin)
        picCanvas_aspect = picCanvas.ClientSize.Height / picCanvas.ClientSize.Width
        If want_aspect > picCanvas_aspect Then
            ' The selected area is too tall and thin.
            ' Make it wider.
            wid = (m_Wymax - m_Wymin) / picCanvas_aspect
            mid = (m_Wxmin + m_Wxmax) / 2
            m_Wxmin = mid - wid / 2
            m_Wxmax = mid + wid / 2
        Else
            ' The selected area is too short and wide.
            ' Make it taller.
            hgt = (m_Wxmax - m_Wxmin) * picCanvas_aspect
            mid = (m_Wymin + m_Wymax) / 2
            m_Wymin = mid - hgt / 2
            m_Wymax = mid + hgt / 2
        End If
    End Sub

#End Region ' Aspect Code

#Region "Mandelbrot Drawing code"

    ' The BackgroundWorkers.
    Private Const NUM_WORKERS As Integer = 2
    Private m_Workers(0 To NUM_WORKERS - 1) As BackgroundWorker
    Private m_NumWorkersRunning As Integer = 0

    ' Draw the Mandelbrot set.
    Private Sub DrawMandelbrot()
        ' Do nothing if the canvas has zero size.
        If picCanvas.ClientSize.Width < 1 Or _
           picCanvas.ClientSize.Height < 1 Then Exit Sub

        ' Prepare the UI.
        btnStop.Enabled = True
        btnFullScale.Enabled = False
        picCanvas.Cursor = Cursors.WaitCursor
        Me.Cursor = Cursors.WaitCursor
        Me.Text = "Working..."
        Me.Refresh()

        ' Make the result bitmap.
        Dim wid As Integer = picCanvas.ClientSize.Width
        Dim hgt As Integer = picCanvas.ClientSize.Height
        m_Bitmap = New Bitmap(wid, hgt)
        Using gr As Graphics = Graphics.FromImage(m_Bitmap)
            gr.Clear(Color.LightGray)
        End Using
        picCanvas.Image = m_Bitmap

        ' Prepare for calculations.
        ' Adjust the coordinate bounds to fit picCanvas.
        AdjustAspect()

        ' dReaC is the change in the real part
        ' (X value) for C. dImaC is the change in the
        ' imaginary part (Y value).
        Dim dReaC, dImaC As Double
        dReaC = (m_Wxmax - m_Wxmin) / (wid - 1)
        dImaC = (m_Wymax - m_Wymin) / (hgt - 1)

        ' We are about to start the workers.
        ' Record the number started so we can wait
        ' for thaht many to finish.
        m_NumWorkersRunning = NUM_WORKERS

        ' Create the workers.
        Dim xmin As Integer = 0
        Dim dxmin As Integer = wid \ NUM_WORKERS
        For i As Integer = 0 To NUM_WORKERS - 1
            ' Prepare the worker.
            m_Workers(i) = New BackgroundWorker
            m_Workers(i).WorkerReportsProgress = True
            m_Workers(i).WorkerSupportsCancellation = True

            ' Install the worker's event handlers.
            AddHandler m_Workers(i).ProgressChanged, AddressOf Worker_ProgressChanged
            AddHandler m_Workers(i).RunWorkerCompleted, AddressOf Worker_RunWorkerCompleted
            AddHandler m_Workers(i).DoWork, AddressOf Worker_DoWork

            ' Get the bounds of the area that this worker will calculate.
            Dim xmax As Integer = xmin + dxmin - 1
            ' If it's the last worker, make it do the rest.
            If i = NUM_WORKERS - 1 Then xmax = wid - 1

            ' Prepare the worker's assignment.
            Dim worker_info As New WorkerInfo( _
                Me, xmin, xmax, 0, hgt - 1, dReaC, dImaC)

            ' Start the worker.
            'Debug.WriteLine("")
            'Debug.WriteLine("Starting worker " & i & ": " & worker_info.xmin & " - " & worker_info.xmax)
            m_Workers(i).RunWorkerAsync(worker_info)
            xmin += dxmin - 1
        Next i
    End Sub

#End Region ' Mandelbrot Drawing code

#Region "Worker Code"

    ' Draw the Mandelbrot set.
    Private Sub Worker_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs)
        ' Work until the magnitude squared > 4.
        Const MAX_MAG_SQUARED As Double = 4.0

        ' Lower the thread's priority so the Stop button can fire.
        Thread.CurrentThread.Priority = ThreadPriority.BelowNormal

        ' Get this BackgroundWorker.
        Dim my_worker As BackgroundWorker = DirectCast(sender, BackgroundWorker)

        ' Get our assignment.
        Dim worker_info As WorkerInfo = DirectCast(e.Argument, WorkerInfo)
        Dim xmin As Integer = worker_info.xmin
        Dim xmax As Integer = worker_info.xmax
        Dim ymin As Integer = worker_info.ymin
        Dim ymax As Integer = worker_info.ymax
        Dim dReaC As Double = worker_info.dReaC
        Dim dImaC As Double = worker_info.dImaC
        'Debug.WriteLine("Worker started: " & worker_info.xmin & " - " & worker_info.xmax)

        ' Create the result bitmap.
        Dim bm As New Bitmap(xmax - xmin + 1, ymax - ymin + 1)
        worker_info.Result = bm
        e.Result = worker_info

        Using gr As Graphics = Graphics.FromImage(bm)
            ' Calculate the values.
            Dim clr As Integer
            Dim ReaC, ImaC, ReaZ, ImaZ, ReaZ2, ImaZ2 As Double
            ReaC = m_Wxmin + xmin * dreaC
            For i As Integer = xmin To xmax
                ImaC = m_Wymin + ymin * dImaC
                For j As Integer = ymin To ymax
                    ReaZ = 0
                    ImaZ = 0
                    ReaZ2 = 0
                    ImaZ2 = 0
                    clr = 0
                    Do While clr < MaxIterations AndAlso _
                            ReaZ2 + ImaZ2 < MAX_MAG_SQUARED
                        ' Calculate Z(clr).
                        ReaZ2 = ReaZ * ReaZ
                        ImaZ2 = ImaZ * ImaZ
                        ImaZ = 2 * ImaZ * ReaZ + ImaC
                        ReaZ = ReaZ2 - ImaZ2 + ReaC
                        clr = clr + 1
                    Loop

                    ' Set the pixel's color.
                    SyncLock m_Bitmap
                        bm.SetPixel(i - xmin, j - ymin, m_Colors(clr Mod m_NumColors))
                    End SyncLock

                    ' Move to the next point.
                    ImaC = ImaC + dImaC
                Next j
                ReaC = ReaC + dReaC

                If i Mod 10 = 0 Then
                    ' Report our progress.
                    Dim progress As Integer = CInt(100 * (i - xmin) / (xmax - xmin))
                    'Debug.WriteLine("Reporting progress " & progress & ": " & worker_info.xmin & " - " & worker_info.xmax)
                    my_worker.ReportProgress(progress, worker_info)

                    ' See if we should cancel.
                    'Thread.Sleep(1) ' Not needed if the priority is lowered.
                    If my_worker.CancellationPending Then
                        e.Cancel = True
                        Exit Sub
                    End If
                End If
            Next i
        End Using ' gr
    End Sub

    ' Display the progress.
    Private Sub Worker_ProgressChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs)
        ' Display this piece of the image.
        Dim worker_info As WorkerInfo = DirectCast(e.UserState, WorkerInfo)
        Using gr As Graphics = Graphics.FromImage(m_Bitmap)
            'Needed? SyncLock m_Bitmap
            gr.DrawImage(worker_info.Result, worker_info.xmin, worker_info.ymin)
            'Needed? End SyncLock
        End Using

        picCanvas.Refresh()
    End Sub

    ' The wroker is done. Display the new image.
    Private Sub Worker_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs)
        ' See if the worker canceled.
        If e.Cancelled Then
            ' Trying to access e.Result for a worker that canceled
            ' throws an InvalidOperationException.
        Else
            ' Display this piece of the image.
            Dim worker_info As WorkerInfo = DirectCast(e.Result, WorkerInfo)
            Using gr As Graphics = Graphics.FromImage(m_Bitmap)
                'Needed? SyncLock m_Bitmap
                gr.DrawImage(worker_info.Result, worker_info.xmin, worker_info.ymin)
                'Needed? End SyncLock
            End Using

            picCanvas.Refresh()
        End If

        ' See if all of the workers are finished.
        m_NumWorkersRunning -= 1
        If m_NumWorkersRunning = 0 Then
            ' All the workers are finished.
            ' Reset the UI.
            btnStop.Enabled = False
            btnFullScale.Enabled = True
            picCanvas.Cursor = Cursors.Cross
            Me.Cursor = Cursors.Default
            Me.Text = "Mandelbrot (" & _
                Format$(m_Wxmin) & ", " & _
                Format$(m_Wymin) & ")-(" & _
                Format$(m_Wxmax) & ", " & _
                Format$(m_Wymax) & ")"

            ' Dispose of the workers.
            For i As Integer = 0 To NUM_WORKERS - 1
                m_Workers(i).Dispose()
                m_Workers(i) = Nothing
            Next i
        End If
    End Sub

#End Region ' Worker Code

End Class

' Information that describes the part of the
' Mandelbrot Set that a BackgroundWorker should build.
Public Class WorkerInfo
    Public MyForm As Form1
    Public xmin As Integer
    Public xmax As Integer
    Public ymin As Integer
    Public ymax As Integer
    Public dReaC As Double
    Public dImaC As Double
    Public Result As Bitmap

    Public Sub New(ByVal new_MyForm As Form1, ByVal new_xmin As Integer, ByVal new_xmax As Integer, ByVal new_ymin As Integer, ByVal new_ymax As Integer, ByVal new_dReaC As Double, ByVal new_dImaC As Double)
        MyForm = new_MyForm
        xmin = new_xmin
        xmax = new_xmax
        ymin = new_ymin
        ymax = new_ymax
        dReaC = new_dReaC
        dImaC = new_dImaC
    End Sub
End Class